#ifndef CALLBACK_CPP
#define CALLBACK_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Callback
//
// BASE CLASSES: None.
//
// PURPOSE: Graphical User Interface callback functions.
//
// AUTHOR: Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Callback.hpp"
#include "DataPaths/Monocycle/MonocycleDataPath.hpp"

//Constants

//Externals

extern FILE* yyin;
extern int parse(void);
extern unsigned int yyerrcount;
extern DataPath* DP;
extern Memory yyInstrMem;
extern word wNewPC;
extern word wOldPC;

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnLoad
//
// DESCRIPTION: This function is called when the 'Load File' button is pressed.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnLoad(void)
{ 
    GtkWidget* pLoadFile;
    const gchar* szPath;

    /* We create the window */
    pLoadFile = gtk_file_selection_new("Load a MIPS assembly file...");

    /* Set window type to MODAL */
    gtk_window_set_modal(GTK_WINDOW(pLoadFile),TRUE);
    
    /* Display the window */
    switch(gtk_dialog_run(GTK_DIALOG(pLoadFile)))
      {
      case GTK_RESPONSE_OK:
	szPath =
	  gtk_file_selection_get_filename(GTK_FILE_SELECTION(pLoadFile));
	yyin = fopen(szPath,"r");
	if(yyin == (FILE*)0)
	  {
	    gtk_label_set_text(GTK_LABEL(pLabel),"Can\'t open file !");
	  }
	else
	  {
	    Callback::OnReset();
	    parse();
	    fclose(yyin);
	    AllocTable* pInstrTable;
	    pInstrTable = yyInstrMem.GetAllocTable();
	    DP->Init(pInstrTable,(AllocTable*)0);
	    if(yyerrcount == 0)
	      {
		gtk_label_set_text(GTK_LABEL(pLabel),
				   "File loaded. Click the \'Next Cycle\'"
				   " button to start.");
	      }
	    else
	      {
		gtk_label_set_text(GTK_LABEL(pLabel),
				   "Your file contains errors. Click the "
				   "\'!\' button to see a detailed error "
				   "report.");
	      }
	  }
	break;
      default:
	break;
      }
    gtk_widget_destroy(pLoadFile);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnNext
//
// DESCRIPTION: This function is called when the 'Next Cycle' button is pressed
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnNext(void)
{
  if(START_ADDR < END_ADDR) // Memory not empty
    {
      if((wNewPC >= START_ADDR) && (wNewPC < END_ADDR))
	{
	  wOldPC = wNewPC;
	  wNewPC = DP->Cycle();
	}
      else
	{
	  if((wOldPC + 0x00000004) < END_ADDR) // Virtual SIGSEGV !
	    {
	      char szTmp[86];
	      snprintf(szTmp,86,
		       "BAD ADDRESS: <tt>0x%08X</tt>. "
		       "DataPath Reset. "
		       "Please load a MIPS assembly file..."
		       ,(unsigned int)wNewPC);
	      Callback::OnReset();
	      gtk_label_set_markup(GTK_LABEL(pLabel),szTmp);
	    }
	  else
	    {
	      gtk_label_set_text(GTK_LABEL(pLabel),
				 "No more instruction to display. "
				 "Please load a MIPS assembly file...");
	    }
	}
    }
  else // Memory empty
    {
      gtk_label_set_text(GTK_LABEL(pLabel),"No file loaded! "
			 "Please load a MIPS assembly file...");
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnReset
//
// DESCRIPTION: This function is called when the 'Reset' button is pressed.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnReset(void)
{
  delete DP; // Most effective...
  yyInstrMem.Reset();
  gtk_text_buffer_set_text(pErr,"",-1); // Delete parser output
  wOldPC = START_ADDR;
  wNewPC = START_ADDR;
  END_ADDR = START_ADDR;
  DP = new MonocycleDataPath(DISPLAY_GRAPHIC);
  DP->Display();
  gtk_label_set_text(GTK_LABEL(pLabel),"DataPath Reset. "
		     "Please load a MIPS assembly file...");
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnError
//
// DESCRIPTION: This function is called when the '!' button is pressed.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnError(void)
{
  GtkWidget* pErrWin;
  GtkWidget* pScroll;
  GtkWidget* pView;
  
  pErrWin = gtk_dialog_new_with_buttons("The Koyote Project - Parser output",
					NULL,
					GTK_DIALOG_MODAL,
					GTK_STOCK_CLOSE,
					GTK_RESPONSE_CLOSE,
					NULL);
  gtk_widget_set_size_request(pErrWin,600,400);
  
  pView = gtk_text_view_new_with_buffer(pErr);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(pView),FALSE);
  pScroll = gtk_scrolled_window_new(NULL,NULL);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(pErrWin)->vbox),pScroll);
  gtk_container_add(GTK_CONTAINER(pScroll),pView);
  gtk_widget_show_all(pErrWin);
  
  gtk_dialog_run(GTK_DIALOG(pErrWin));
  gtk_widget_destroy(pErrWin);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnQuit
//
// DESCRIPTION: This function is called when the 'Quit' button is pressed.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnQuit(void)
{
  GtkWidget *pQuestion;
  pQuestion = gtk_message_dialog_new(GTK_WINDOW(pWindow),
				     GTK_DIALOG_MODAL,
				     GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_YES_NO,
				     "Do  you  really  want  to\n"
				     "quit The Koyote Project ?");
  
  switch(gtk_dialog_run(GTK_DIALOG(pQuestion)))
    {
    case GTK_RESPONSE_YES:
      gtk_main_quit();
      break;
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_NO:
      gtk_widget_destroy(pQuestion);
      break;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: OnAbout
//
// DESCRIPTION: This function displays the 'About' window.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Callback::OnAbout(void)
{
    GtkWidget* pAbout;

    pAbout = gtk_message_dialog_new(GTK_WINDOW(pWindow),
				    GTK_DIALOG_MODAL,
				    GTK_MESSAGE_INFO,
				    GTK_BUTTONS_OK,
				    "The KOYOTE Project - KOYOTE Version "
				    KOYOTE_VERSION
				    ".\n\nCopyright (C) 2003-2004 "
				    "Nicolas MOREAU & Brice DONVAL.\n"
				    "Visit http://koyoteproject.free.fr "
				    "for updates.\n\n"
				    "This is FREE SOFTWARE; "
				    "see the source for copying conditions.\n"
				    "There is NO WARRANTY; not even for "
				    "MERCHANTABILITY or FITNESS FOR A "
				    "PARTICULAR PURPOSE.");
    gtk_window_set_title(GTK_WINDOW(pAbout),"About...");
    gtk_dialog_run(GTK_DIALOG(pAbout));
    gtk_widget_destroy(pAbout);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ExposeEvent
//
// DESCRIPTION: Fills the screen with the background pixmap.
//
// PARAMETERS: GTK Widget, GDK EventExpose
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint
Callback::ExposeEvent(GtkWidget* pWidget,GdkEventExpose* pEvent)
{
  gdk_draw_pixmap(pWidget->window,
                  pWidget->style->fg_gc[GTK_WIDGET_STATE(pWidget)],
                  pPixmap,
                  pEvent->area.x, pEvent->area.y,
                  pEvent->area.x, pEvent->area.y,
                  pEvent->area.width, pEvent->area.height);

  return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ConfigureEvent
//
// DESCRIPTION: Creates a new background pixmap.
//
// PARAMETERS: GTK Widget, GDK EventConfigure.
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint
Callback::ConfigureEvent(GtkWidget* pWidget,GdkEventConfigure* pEvent)
{
  if(pPixmap != NULL)
    gdk_pixmap_unref(pPixmap);

  pPixmap = gdk_pixmap_new(pWidget->window,
			   pWidget->allocation.width,
			   pWidget->allocation.height,
			   -1);

  /* White background */
  gdk_draw_rectangle(pPixmap,
		     pWidget->style->white_gc,
		     TRUE,
		     0, 0,
		     pWidget->allocation.width,
		     pWidget->allocation.height);

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MotionNotifyEvent
//
// DESCRIPTION: Mouse move handler. - DEPRECATED
//
// PARAMETERS: GTK Widget, GDK EventMotion.
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint
Callback::MotionNotifyEvent(GtkWidget* pWidget,GdkEventMotion* pEvent)
{
  return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ButtonPressEvent
//
// DESCRIPTION: Mouse button handler.
//
// PARAMETERS: GTK Widget, GDK EventButton.
//
// RETURNS: gint.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
gint
Callback::ButtonPressEvent(GtkWidget* pWidget,GdkEventButton* pEventButton)
{
  MouseEvent eMouseEvent;
  Coordinate* pCoord = new Coordinate;
  pCoord->X = (int)pEventButton->x;
  pCoord->Y = (int)pEventButton->y;
  switch(pEventButton->type)
    {
    default:
      eMouseEvent = MOUSE_CLK;
    }
  DP->MouseAction(pCoord,eMouseEvent);
  delete pCoord;
  return FALSE;
}

#endif // CALLBACK_CPP
