#ifndef REGISTERBOX_CPP
#define REGISTERBOX_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: RegisterBox
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple RegisterBox object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "RegisterBox.hpp"
#include "XPM/RegisterBox_disabled.xpm"
#include "XPM/RegisterBox_enabled.xpm"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: RegisterBox
//
// DESCRIPTION: Constructor for RegisterBox objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
RegisterBox::RegisterBox(Coordinate* pCoord,
			 const int nCoord,
			 const DisplayMode eDisplayMode)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_nValue = 2;
  m_pwValue = new word[m_nValue];
  m_pwRegister = new word[MAX_REGISTERS];
  Reset();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~RegisterBox
//
// DESCRIPTION: Destructor for RegisterBox objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
RegisterBox::~RegisterBox(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
RegisterBox::GetValue(const int nIndex,const AskMode eAskMode)
{
  word ReadReg1 = 0x00000000;
  word ReadReg2 = 0x00000000;
  switch(nIndex)
    {
    case 0:
    case 1:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      if(nIndex == 0) // Rs
	{
	  ReadReg1 = m_ppInput[0]->GetValue(m_pnOutput[0],eAskMode);
	  ReadReg2 = m_ppInput[1]->GetValue(m_pnOutput[1],ASK_PASV);
	}
      else if(nIndex == 1) // Rt
	{
	  ReadReg1 = m_ppInput[0]->GetValue(m_pnOutput[0],ASK_PASV);
	  ReadReg2 = m_ppInput[1]->GetValue(m_pnOutput[1],eAskMode);
	}
      m_pwValue[0] = m_pwRegister[ReadReg1];
      m_pwValue[1] = m_pwRegister[ReadReg2];
      if(m_eIsUpToDate == false)
	{
	  word RegWrite = m_ppInput[4]->GetValue(m_pnOutput[4],eAskMode);
	  m_eIsUpToDate = true; // Prevents loop
	  if(Core::Bit(RegWrite,0) == 0x01)
	    {
	      m_eIsActivated = true; // If we need to write, we are active !
	      word WriteReg  = m_ppInput[2]->GetValue(m_pnOutput[2],ASK_ACTV);
	      word WriteData = m_ppInput[3]->GetValue(m_pnOutput[3],ASK_ACTV);
	      if(m_eDisplayMode == DISPLAY_GRAPHIC) // Graphic Mode
		m_ppInput[4]->GetValue(m_pnOutput[4],ASK_ACTV); // Update
	      if(WriteReg != 0) // Register $0 cannot be written !
		m_pwRegister[WriteReg] = WriteData;
	    }
	  else
	    {
	      m_ppInput[2]->GetValue(0,ASK_PASV);
	      m_ppInput[3]->GetValue(0,ASK_PASV);
	    }
	}
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This public method is used to reset the object.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
RegisterBox::Reset(void)
{
  // We reset the outputs :
  m_pwValue[0] = 0x00000000;
  m_pwValue[1] = 0x00000000;
  // We reset the registers :
  m_pwRegister[r0] = 0x00000000;
  m_pwRegister[at] = 0x00000000;
  m_pwRegister[v0] = 0x00000000;
  m_pwRegister[v1] = 0x00000000;
  m_pwRegister[a0] = 0x00000000;
  m_pwRegister[a1] = 0x00000000;
  m_pwRegister[a2] = 0x00000000;
  m_pwRegister[a3] = 0x00000000;
  m_pwRegister[t0] = 0x00000000;
  m_pwRegister[t1] = 0x00000000;
  m_pwRegister[t2] = 0x00000000;
  m_pwRegister[t3] = 0x00000000;
  m_pwRegister[t4] = 0x00000000;
  m_pwRegister[t5] = 0x00000000;
  m_pwRegister[t6] = 0x00000000;
  m_pwRegister[t7] = 0x00000000;
  m_pwRegister[s0] = 0x00000000;
  m_pwRegister[s1] = 0x00000000;
  m_pwRegister[s2] = 0x00000000;
  m_pwRegister[s3] = 0x00000000;
  m_pwRegister[s4] = 0x00000000;
  m_pwRegister[s5] = 0x00000000;
  m_pwRegister[s6] = 0x00000000;
  m_pwRegister[s7] = 0x00000000;
  m_pwRegister[t8] = 0x00000000;
  m_pwRegister[t9] = 0x00000000;
  m_pwRegister[k0] = 0x00000000;
  m_pwRegister[k1] = 0x00000000;
  m_pwRegister[gp] = 0x10008000;
  m_pwRegister[sp] = 0x7FFFEFFC;
  m_pwRegister[s8] = 0x00000000;
  m_pwRegister[ra] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
RegisterBox::Display(void)
{
  GdkPixmap* pPixmapTmp = NULL;
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      if(m_eIsActivated == true)
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)RegisterBox_enabled);
	}
      else
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)RegisterBox_disabled);
	}
      gdk_draw_drawable(pPixmap,
			pDrawingArea->style->fg_gc[GTK_WIDGET_STATE(pDrawingArea)],
			pPixmapTmp,0,0,
			m_pCoord[0].X,
			m_pCoord[0].Y,
			m_pCoord[1].X - m_pCoord[0].X,
			m_pCoord[1].Y - m_pCoord[0].Y);
      gdk_pixmap_unref(pPixmapTmp);
      gtk_widget_queue_draw_area (pDrawingArea,
				  0,0,
				  pDrawingArea->allocation.width,
				  pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:
      using namespace std;
      cout << hex << setfill('0') << internal << uppercase ;
      for(int r = 0 ; r < (MAX_REGISTERS/4) ; r++)
	{
	  cout << szRegister[r]    << " = 0x";
	  cout << setw(8) << m_pwRegister[r]    << "  ";
	  cout << szRegister[r+8]  << " = 0x";
	  cout << setw(8) << m_pwRegister[r+8]  << "  ";
	  cout << szRegister[r+16] << " = 0x";
	  cout << setw(8) << m_pwRegister[r+16] << "  ";
	  cout << szRegister[r+24] << " = 0x";
	  cout << setw(8) << m_pwRegister[r+24] << endl;
	}
      break;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseAction
//
// DESCRIPTION: This public method is called when a mouse event occurs.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
RegisterBox::MouseAction(const MouseEvent eMouseEvent)
{
  if(m_eIsUpToDate == true)
    switch(eMouseEvent)
      {
      default:
      case MOUSE_CLK: // Mouse Clicked
	
	GtkWidget* pPopup = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_position(GTK_WINDOW(pPopup),GTK_WIN_POS_MOUSE);
	gtk_window_set_resizable(GTK_WINDOW(pPopup),FALSE);
	g_signal_connect(G_OBJECT(pPopup),"destroy",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	char szVal[570];
	snprintf(szVal,570,
		 "<tt><i>Register values:\n</i>"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X\n"
		 "%s = 0x%08X  %s = 0x%08X  %s = 0x%08X  %s = 0x%08X</tt>",
		 szRegister[r0],(unsigned int)m_pwRegister[r0],
		 szRegister[t0],(unsigned int)m_pwRegister[t0],
		 szRegister[s0],(unsigned int)m_pwRegister[s0],
		 szRegister[t8],(unsigned int)m_pwRegister[t8],
		 szRegister[at],(unsigned int)m_pwRegister[at],
		 szRegister[t1],(unsigned int)m_pwRegister[t1],
		 szRegister[s1],(unsigned int)m_pwRegister[s1],
		 szRegister[t9],(unsigned int)m_pwRegister[t9],
		 szRegister[v0],(unsigned int)m_pwRegister[v0],
		 szRegister[t2],(unsigned int)m_pwRegister[t2],
		 szRegister[s2],(unsigned int)m_pwRegister[s2],
		 szRegister[k0],(unsigned int)m_pwRegister[k0],
		 szRegister[v1],(unsigned int)m_pwRegister[v1],
		 szRegister[t3],(unsigned int)m_pwRegister[t3],
		 szRegister[s3],(unsigned int)m_pwRegister[s3],
		 szRegister[k1],(unsigned int)m_pwRegister[k1],
		 szRegister[a0],(unsigned int)m_pwRegister[a0],
		 szRegister[t4],(unsigned int)m_pwRegister[t4],
		 szRegister[s4],(unsigned int)m_pwRegister[s4],
		 szRegister[gp],(unsigned int)m_pwRegister[gp],
		 szRegister[a1],(unsigned int)m_pwRegister[a1],
		 szRegister[t5],(unsigned int)m_pwRegister[t5],
		 szRegister[s5],(unsigned int)m_pwRegister[s5],
		 szRegister[sp],(unsigned int)m_pwRegister[sp],
		 szRegister[a2],(unsigned int)m_pwRegister[a2],
		 szRegister[t6],(unsigned int)m_pwRegister[t6],
		 szRegister[s6],(unsigned int)m_pwRegister[s6],
		 szRegister[s8],(unsigned int)m_pwRegister[s8],
		 szRegister[a3],(unsigned int)m_pwRegister[a3],
		 szRegister[t7],(unsigned int)m_pwRegister[t7],
		 szRegister[s7],(unsigned int)m_pwRegister[s7],
		 szRegister[ra],(unsigned int)m_pwRegister[ra]
		 );
	
	GtkWidget* pVal = gtk_label_new(NULL);
	GtkWidget* pEventBox = gtk_event_box_new();
	gtk_label_set_markup(GTK_LABEL(pVal),szVal);
	
	g_signal_connect(G_OBJECT(pPopup),"leave-notify-event",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	g_signal_connect(G_OBJECT(pPopup),"button-press-event",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	
	gtk_container_add(GTK_CONTAINER(pEventBox),pVal);
	gtk_container_add(GTK_CONTAINER(pPopup),pEventBox);
	gtk_widget_show_all(pPopup);
	
	break;
      }
}

#endif // REGISTERBOX_CPP
