#ifndef MEMORY_CPP
#define MEMORY_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Memory
//
// BASE CLASSES: None.
//
// PURPOSE: This file describes the MIPS Memory.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Memory.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Memory
//
// DESCRIPTION: Constructor for Memory objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Memory::Memory(void)
{
  m_pAllocTable = (AllocTable*)0;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~Memory
//
// DESCRIPTION: Destructor for Memory objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Memory::~Memory(void)
{
  Reset();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Init
//
// DESCRIPTION: This public method initializes the memory allocation table
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Memory::Init(AllocTable* pAllocTable)
{
  if(m_pAllocTable != (AllocTable*)0) // Need memory reset
    Reset();
  m_pAllocTable = pAllocTable;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetAllocTable
//
// DESCRIPTION: This public method returns the memory allocation table
//              Useful to display memory content
//
// PARAMETERS: None.
//
// RETURNS: Allocation table : a AllocTable (Address <-> Content) list.
//          Please refer to AllocTable.hpp for more details.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
AllocTable*
Memory::GetAllocTable(void)
{
  return m_pAllocTable;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This public method resets Memory objects.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Memory::Reset(void)
{
  if(m_pAllocTable != (AllocTable*)0) // This is not the last list element
    {
      delete m_pAllocTable;
      m_pAllocTable = (AllocTable*)0;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: operator[]
//
// DESCRIPTION: Array-access operator for Memory objects.
//
// PARAMETERS: Memory address (word).
//
// RETURNS: Memory content, if memory at wAddress has been initialized ;
//          Random word, else.
//
// SIDE EFFECTS: Calling operator[] allocates a new element if address doesn't
//               exist. This means that if you call operator[] to read a value
//               which has not been initialized, operator[] will create it for
//               you and initialize it with random value.
//
///////////////////////////////////////////////////////////////////////////////
word&
Memory::operator[](word wAddress)
{
  wAddress = Core::Sub(wAddress,31,2); // Align
  if(m_pAllocTable == (AllocTable*)0)
    m_pAllocTable = new AllocTable(wAddress);
  AllocTable* pAllocTable = m_pAllocTable;
  while(pAllocTable->m_wAddress != wAddress) // We didn't find the address
  {
    if(pAllocTable->m_pNext == (AllocTable*)0) // This is the last element
      {
	// Allocate and insert new element at current position
	pAllocTable->m_pNext = new AllocTable(wAddress);
	pAllocTable = pAllocTable->m_pNext; // Go to next element
      }
    else // This is not the last element
      {
	if(pAllocTable->m_pNext->m_wAddress > wAddress) // Should stop here
	  {
	    AllocTable* pTmp = pAllocTable->m_pNext;
	    // Allocate and insert new element at current position
	    pAllocTable->m_pNext = new AllocTable(wAddress);
	    pAllocTable = pAllocTable->m_pNext; // Go to next element
	    pAllocTable->m_pNext = pTmp;
	  }
	else // Must continue searching, this is not the place
	  {
	    pAllocTable = pAllocTable->m_pNext; // Go to next element
	  }
      }
  }
  return pAllocTable->m_wContent; // Memory content
}

#endif // MEMORY_CPP
