#ifndef SHIFTER32_CPP
#define SHIFTER32_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: SHIFTER32
//
// BASE CLASSES: None.
//
// PURPOSE: A simple 32-bits shift unit.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "SHIFTER32.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Result
//
// DESCRIPTION: This public method is used to get the result of the shift unit.
//
// PARAMETERS: 
//  - w : The word to shift.
//  - shamt : The shift amount
//  - shiftOp : The shift operation. Legal values are :
//          * SRL  (0x00)
//          * SLL  (0x01)
//          * SRA  (0x02)
//          * SRLV (0x04)
//          * SLLV (0x05)
//          * SRAV (0x06)
//
// RETURNS: The Result output of the shift unit.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
SHIFTER32::Result(word w,word shamt,byte shiftOp)
{ 
  shamt=Core::Sel(shamt,4,0); //We extract the 5 significant bits we need
  return Core::Mux(w>>shamt,w<<shamt,(w>>shamt)|Core::Sub(Core::Mux(0x00000000,0xFFFFFFFF,Core::Bit(w,31)),31,31-shamt),Core::Sel(shiftOp,1,0));
}

#endif // SHIFTER32_CPP
