#ifndef DATAPATH_CPP
#define DATAPATH_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: DataPath
//
// BASE CLASSES: None.
//
// PURPOSE: This generic class describes a simple datapath.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "DataPath.hpp"

//Constants
 
//Externals
 
//Types and classes
 
//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DataPath
//
// DESCRIPTION: Constructor for DataPath objects.
//
// PARAMETERS: Display Mode (Graphic or Text-based).
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataPath::DataPath(DisplayMode eDisplayMode)
{
  m_eDisplayMode = eDisplayMode;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~DataPath
//
// DESCRIPTION: Destructor for DataPath objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataPath::~DataPath(void)
{
  // Nothing to do
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the datapath.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPath::Display(void)
{
  int nComponent;
  for(nComponent = 0 ; nComponent < m_nComponent ; nComponent++)
    m_ppComponent[nComponent]->Display();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This method is used to reset the datapath.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPath::Reset(void)
{
  int nComponent;
  for(nComponent = 0 ; nComponent < m_nComponent ; nComponent++)
    m_ppComponent[nComponent]->Reset();
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseAction
//
// DESCRIPTION: This method is called when a mouse event occurs.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPath::MouseAction(Coordinate* pCoord,const MouseEvent eMouseEvent)
{
  int nComponent;
  for(nComponent = 0 ; nComponent < m_nComponent ; nComponent++)
    if(m_ppComponent[nComponent]->MouseIsOver(pCoord) == true)
      {
	m_ppComponent[nComponent]->MouseAction(eMouseEvent);
	return;
      }
}

#endif // DATAPATH_CPP
