#ifndef MONOCYCLEDATAPATH_CPP
#define MONOCYCLEDATAPATH_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: MonocycleDataPath
//
// BASE CLASSES: DataPath.
//
// PURPOSE: This class describes a simple monocycle datapath.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "MonocycleDataPath.hpp"

//Constants

#define MAX_MONOCYCLE_COMPONENTS 127

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MonocycleDataPath
//
// DESCRIPTION: Constructor for MonocycleDataPath objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
MonocycleDataPath::MonocycleDataPath(DisplayMode eDisplayMode)
  : DataPath(eDisplayMode)
{
  // Allocate enough space for all component and cordinate tables :
  
  m_nComponent = MAX_MONOCYCLE_COMPONENTS;
  m_ppComponent = new DataPathComponent* [m_nComponent];
  m_ppCoord = new Coordinate* [m_nComponent];

  // Let's instantiate all the components !

  m_ppCoord[0] = new Coordinate[2];
  m_ppCoord[0][0].X = 40;    m_ppCoord[0][0].Y = 320;
  m_ppCoord[0][1].X = 40+25; m_ppCoord[0][1].Y = 320+73;
  m_ppComponent[0] = new ProgramCounter(m_ppCoord[0],2,eDisplayMode);

  m_ppCoord[1] = new Coordinate[2];
  m_ppCoord[1][0].X = 240;    m_ppCoord[1][0].Y = 130;
  m_ppCoord[1][1].X = 240+49; m_ppCoord[1][1].Y = 130+201;
  m_ppComponent[1] = new ControlUnit(m_ppCoord[1],2,eDisplayMode);

  m_ppCoord[2] = new Coordinate[2];
  m_ppCoord[2][0].X = 820;     m_ppCoord[2][0].Y = 400;
  m_ppCoord[2][1].X = 820+109; m_ppCoord[2][1].Y = 400+109;
  m_ppComponent[2] = new DataMemory(m_ppCoord[2],2,eDisplayMode);

  m_ppCoord[3] = new Coordinate[2];
  m_ppCoord[3][0].X = 110;    m_ppCoord[3][0].Y = 340;
  m_ppCoord[3][1].X = 110+97; m_ppCoord[3][1].Y = 340+97;
  m_ppComponent[3] = new InstructionMemory(m_ppCoord[3],2,eDisplayMode);

  m_ppCoord[4] = new Coordinate[2];
  m_ppCoord[4][0].X = 390;    m_ppCoord[4][0].Y = 330;
  m_ppCoord[4][1].X = 390+85; m_ppCoord[4][1].Y = 330+193;
  m_ppComponent[4] = new RegisterBox(m_ppCoord[4],2,eDisplayMode);

  m_ppCoord[5] = new Coordinate[2];
  m_ppCoord[5][0].X = 290;    m_ppCoord[5][0].Y = 390;
  m_ppCoord[5][1].X = 290+25; m_ppCoord[5][1].Y = 390+60;
  m_ppComponent[5] = new Multiplexor(m_ppCoord[5],2,eDisplayMode,MUX_ASC);

  m_ppCoord[6] = new Coordinate[2];
  m_ppCoord[6][0].X = 340;    m_ppCoord[6][0].Y = 410;
  m_ppCoord[6][1].X = 340+25; m_ppCoord[6][1].Y = 410+60;
  m_ppComponent[6] = new Multiplexor(m_ppCoord[6],2,eDisplayMode,MUX_ASC);

  m_ppCoord[7] = new Coordinate[2];
  m_ppCoord[7][0].X = 340;    m_ppCoord[7][0].Y = 490;
  m_ppCoord[7][1].X = 340+25; m_ppCoord[7][1].Y = 490+60;
  m_ppComponent[7] = new Multiplexor(m_ppCoord[7],2,eDisplayMode,MUX_DSC);

  m_ppCoord[8] = new Coordinate[2];
  m_ppCoord[8][0].X = 480;    m_ppCoord[8][0].Y = 540;
  m_ppCoord[8][1].X = 480+25; m_ppCoord[8][1].Y = 540+60;
  m_ppComponent[8] = new Multiplexor(m_ppCoord[8],2,eDisplayMode,MUX_ASC);

  m_ppCoord[9] = new Coordinate[2];
  m_ppCoord[9][0].X = 510;    m_ppCoord[9][0].Y = 20;
  m_ppCoord[9][1].X = 510+25; m_ppCoord[9][1].Y = 20+60;
  m_ppComponent[9] = new Multiplexor(m_ppCoord[9],2,eDisplayMode,MUX_ASC);

  m_ppCoord[10] = new Coordinate[2];
  m_ppCoord[10][0].X = 560;    m_ppCoord[10][0].Y = 410;
  m_ppCoord[10][1].X = 560+25; m_ppCoord[10][1].Y = 410+60;
  m_ppComponent[10] = new Multiplexor(m_ppCoord[10],2,eDisplayMode,MUX_ASC);

  m_ppCoord[11] = new Coordinate[2];
  m_ppCoord[11][0].X = 560;    m_ppCoord[11][0].Y = 490;
  m_ppCoord[11][1].X = 560+25; m_ppCoord[11][1].Y = 490+60;
  m_ppComponent[11] = new Multiplexor(m_ppCoord[11],2,eDisplayMode,MUX_ASC);

  m_ppCoord[12] = new Coordinate[2];
  m_ppCoord[12][0].X = 590;    m_ppCoord[12][0].Y = 578;
  m_ppCoord[12][1].X = 590+25; m_ppCoord[12][1].Y = 578+60;
  m_ppComponent[12] = new Multiplexor(m_ppCoord[12],2,eDisplayMode,MUX_DSC);

  m_ppCoord[13] = new Coordinate[2];
  m_ppCoord[13][0].X = 750;    m_ppCoord[13][0].Y = 330;
  m_ppCoord[13][1].X = 750+25; m_ppCoord[13][1].Y = 330+60;
  m_ppComponent[13] = new Multiplexor(m_ppCoord[13],2,eDisplayMode,MUX_DSC);

  m_ppCoord[14] = new Coordinate[2];
  m_ppCoord[14][0].X = 850;    m_ppCoord[14][0].Y = 300;
  m_ppCoord[14][1].X = 850+25; m_ppCoord[14][1].Y = 300+60;
  m_ppComponent[14] = new Multiplexor(m_ppCoord[14],2,eDisplayMode,MUX_DSC);

  m_ppCoord[15] = new Coordinate[2];
  m_ppCoord[15][0].X = 830;    m_ppCoord[15][0].Y = 60;
  m_ppCoord[15][1].X = 830+25; m_ppCoord[15][1].Y = 60+60;
  m_ppComponent[15] = new Multiplexor(m_ppCoord[15],2,eDisplayMode,MUX_ASC);

  m_ppCoord[16] = new Coordinate[2];
  m_ppCoord[16][0].X = 880;    m_ppCoord[16][0].Y = 40;
  m_ppCoord[16][1].X = 880+25; m_ppCoord[16][1].Y = 40+60;
  m_ppComponent[16] = new Multiplexor(m_ppCoord[16],2,eDisplayMode,MUX_DSC);

  m_ppCoord[17] = new Coordinate[2];
  m_ppCoord[17][0].X = 950;    m_ppCoord[17][0].Y = 480;
  m_ppCoord[17][1].X = 950+25; m_ppCoord[17][1].Y = 480+60;
  m_ppComponent[17] = new Multiplexor(m_ppCoord[17],2,eDisplayMode,MUX_DSC);

  m_ppCoord[18] = new Coordinate[1];
  m_ppCoord[18][0].X = 0; m_ppCoord[18][0].Y = 0; ////**////
  m_ppComponent[18] = new Constant(m_ppCoord[18],1,eDisplayMode,0x00000004);

  m_ppCoord[19] = new Coordinate[1];
  m_ppCoord[19][0].X = 0; m_ppCoord[19][0].Y = 0; ////**////
  m_ppComponent[19] = new Constant(m_ppCoord[19],1,eDisplayMode,0x0000001F);

  m_ppCoord[20] = new Coordinate[1];
  m_ppCoord[20][0].X = 0; m_ppCoord[20][0].Y = 0; ////**////
  m_ppComponent[20] = new Constant(m_ppCoord[20],1,eDisplayMode,0x00000000);

  m_ppCoord[21] = new Coordinate[2];
  m_ppCoord[21][0].X = 620;    m_ppCoord[21][0].Y = 350;
  m_ppCoord[21][1].X = 620+60; m_ppCoord[21][1].Y = 350+96;
  m_ppComponent[21] = new ALU(m_ppCoord[21],2,eDisplayMode);

  m_ppCoord[22] = new Coordinate[2];
  m_ppCoord[22][0].X = 620;    m_ppCoord[22][0].Y = 530;
  m_ppCoord[22][1].X = 620+60; m_ppCoord[22][1].Y = 530+49;
  m_ppComponent[22] = new Shifter(m_ppCoord[22],2,eDisplayMode);

  m_ppCoord[23] = new Coordinate[2];
  m_ppCoord[23][0].X = 380;    m_ppCoord[23][0].Y = 530;
  m_ppCoord[23][1].X = 380+84; m_ppCoord[23][1].Y = 530+49;
  m_ppComponent[23] = new SignExtender(m_ppCoord[23],2,eDisplayMode);

  m_ppCoord[24] = new Coordinate[2];
  m_ppCoord[24][0].X = 110;    m_ppCoord[24][0].Y = 50;
  m_ppCoord[24][1].X = 110+61; m_ppCoord[24][1].Y = 50+80;
  m_ppComponent[24] = new Adder(m_ppCoord[24],2,eDisplayMode);

  m_ppCoord[25] = new Coordinate[2];
  m_ppCoord[25][0].X = 700;    m_ppCoord[25][0].Y = 80;
  m_ppCoord[25][1].X = 700+61; m_ppCoord[25][1].Y = 80+80;
  m_ppComponent[25] = new Adder(m_ppCoord[25],2,eDisplayMode);

  m_ppCoord[26] = new Coordinate[2];
  m_ppCoord[26][0].X = 400;    m_ppCoord[26][0].Y = 20;
  m_ppCoord[26][1].X = 400+37; m_ppCoord[26][1].Y = 20+25;
  m_ppComponent[26] = new LightShifter(m_ppCoord[26],2,eDisplayMode);

  m_ppCoord[27] = new Coordinate[2];
  m_ppCoord[27][0].X = 620;    m_ppCoord[27][0].Y = 130;
  m_ppCoord[27][1].X = 620+37; m_ppCoord[27][1].Y = 130+25;
  m_ppComponent[27] = new LightShifter(m_ppCoord[27],2,eDisplayMode);

  m_ppCoord[28] = new Coordinate[2];
  m_ppCoord[28][0].X = 820;    m_ppCoord[28][0].Y = 130;
  m_ppCoord[28][1].X = 820+49; m_ppCoord[28][1].Y = 130+26;
  m_ppComponent[28] = new GateAND(m_ppCoord[28],2,eDisplayMode);

  m_ppCoord[29] = new Coordinate[2];
  m_ppCoord[29][0].X = 810;    m_ppCoord[29][0].Y = 300;
  m_ppCoord[29][1].X = 810+25; m_ppCoord[29][1].Y = 300+37;
  m_ppComponent[29] = new GateOR(m_ppCoord[29],2,eDisplayMode);

  m_ppCoord[30] = new Coordinate[2];
  m_ppCoord[30][0].X = 705;    m_ppCoord[30][0].Y = 330;
  m_ppCoord[30][1].X = 705+25; m_ppCoord[30][1].Y = 330+27;
  m_ppComponent[30] = new GateNOT(m_ppCoord[30],2,eDisplayMode);

  m_ppCoord[31] = new Coordinate[2];
  m_ppCoord[31][0].X = 610;    m_ppCoord[31][0].Y = 290;
  m_ppCoord[31][1].X = 610+49; m_ppCoord[31][1].Y = 290+49;
  m_ppComponent[31] = new Comparator(m_ppCoord[31],2,eDisplayMode);

  m_ppCoord[32] = new Coordinate[2];
  m_ppCoord[32][0].X = 65; m_ppCoord[32][0].Y = 360;
  m_ppCoord[32][1].X = 75; m_ppCoord[32][1].Y = 360;
  m_ppComponent[32] = new Wire(m_ppCoord[32],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[33] = new Coordinate[2];
  m_ppCoord[33][0].X = 75;  m_ppCoord[33][0].Y = 360;
  m_ppCoord[33][1].X = 110; m_ppCoord[33][1].Y = 360;
  m_ppComponent[33] = new Wire(m_ppCoord[33],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[34] = new Coordinate[3];
  m_ppCoord[34][0].X = 75;  m_ppCoord[34][0].Y = 360;
  m_ppCoord[34][1].X = 75;  m_ppCoord[34][1].Y = 60;
  m_ppCoord[34][2].X = 110; m_ppCoord[34][2].Y = 60;
  m_ppComponent[34] = new Wire(m_ppCoord[34],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[35] = new Coordinate[2];
  m_ppCoord[35][0].X = 95;  m_ppCoord[35][0].Y = 120;
  m_ppCoord[35][1].X = 110; m_ppCoord[35][1].Y = 120;
  m_ppComponent[35] = new Wire(m_ppCoord[35],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[36] = new Coordinate[2];
  m_ppCoord[36][0].X = 171; m_ppCoord[36][0].Y = 90;
  m_ppCoord[36][1].X = 220; m_ppCoord[36][1].Y = 90;
  m_ppComponent[36] = new Wire(m_ppCoord[36],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[37] = new Coordinate[2];
  m_ppCoord[37][0].X = 207; m_ppCoord[37][0].Y = 390;
  m_ppCoord[37][1].X = 230; m_ppCoord[37][1].Y = 390;
  m_ppComponent[37] = new Wire(m_ppCoord[37],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[38] = new Coordinate[2];
  m_ppCoord[38][0].X = 220; m_ppCoord[38][0].Y = 90;
  m_ppCoord[38][1].X = 450; m_ppCoord[38][1].Y = 90;
  m_ppComponent[38] = new Wire(m_ppCoord[38],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[39] = new Coordinate[3];
  m_ppCoord[39][0].X = 230; m_ppCoord[39][0].Y = 390;
  m_ppCoord[39][1].X = 230; m_ppCoord[39][1].Y = 32;
  m_ppCoord[39][2].X = 400; m_ppCoord[39][2].Y = 32;
  m_ppComponent[39] = new Wire(m_ppCoord[39],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,25,0);

  m_ppCoord[40] = new Coordinate[3];
  m_ppCoord[40][0].X = 220; m_ppCoord[40][0].Y = 90;
  m_ppCoord[40][1].X = 220; m_ppCoord[40][1].Y = 505;
  m_ppCoord[40][2].X = 340; m_ppCoord[40][2].Y = 505;
  m_ppComponent[40] = new Wire(m_ppCoord[40],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[41] = new Coordinate[2];
  m_ppCoord[41][0].X = 230; m_ppCoord[41][0].Y = 390;
  m_ppCoord[41][1].X = 265; m_ppCoord[41][1].Y = 390;
  m_ppComponent[41] = new Wire(m_ppCoord[41],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[42] = new Coordinate[2];
  m_ppCoord[42][0].X = 450; m_ppCoord[42][0].Y = 90;
  m_ppCoord[42][1].X = 450; m_ppCoord[42][1].Y = 32;
  m_ppComponent[42] = new Wire(m_ppCoord[42],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,28);

  m_ppCoord[43] = new Coordinate[2];
  m_ppCoord[43][0].X = 437; m_ppCoord[43][0].Y = 32;
  m_ppCoord[43][1].X = 450; m_ppCoord[43][1].Y = 32;
  m_ppComponent[43] = new Wire(m_ppCoord[43],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,27,0);

  m_ppCoord[44] = new Coordinate[2];
  m_ppCoord[44][0].X = 450; m_ppCoord[44][0].Y = 90;
  m_ppCoord[44][1].X = 665; m_ppCoord[44][1].Y = 90;
  m_ppComponent[44] = new Wire(m_ppCoord[44],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[45] = new Coordinate[2];
  m_ppCoord[45][0].X = 450; m_ppCoord[45][0].Y = 32;
  m_ppCoord[45][1].X = 510; m_ppCoord[45][1].Y = 32;
  m_ppComponent[45] = new Wire(m_ppCoord[45],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[46] = new Coordinate[2];
  m_ppCoord[46][0].X = 665; m_ppCoord[46][0].Y = 90;
  m_ppCoord[46][1].X = 700; m_ppCoord[46][1].Y = 90;
  m_ppComponent[46] = new Wire(m_ppCoord[46],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[47] = new Coordinate[3];
  m_ppCoord[47][0].X = 665; m_ppCoord[47][0].Y = 90;
  m_ppCoord[47][1].X = 665; m_ppCoord[47][1].Y = 70;
  m_ppCoord[47][2].X = 830; m_ppCoord[47][2].Y = 70;
  m_ppComponent[47] = new Wire(m_ppCoord[47],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[48] = new Coordinate[2];
  m_ppCoord[48][0].X = 265; m_ppCoord[48][0].Y = 390;
  m_ppCoord[48][1].X = 265; m_ppCoord[48][1].Y = 380;
  m_ppComponent[48] = new Wire(m_ppCoord[48],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[49] = new Coordinate[2];
  m_ppCoord[49][0].X = 265; m_ppCoord[49][0].Y = 380;
  m_ppCoord[49][1].X = 265; m_ppCoord[49][1].Y = 350;
  m_ppComponent[49] = new Wire(m_ppCoord[49],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[50] = new Coordinate[2];
  m_ppCoord[50][0].X = 265; m_ppCoord[50][0].Y = 350;
  m_ppCoord[50][1].X = 265; m_ppCoord[50][1].Y = 331;
  m_ppComponent[50] = new Wire(m_ppCoord[50],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[51] = new Coordinate[2];
  m_ppCoord[51][0].X = 265; m_ppCoord[51][0].Y = 380;
  m_ppCoord[51][1].X = 275; m_ppCoord[51][1].Y = 380;
  m_ppComponent[51] = new Wire(m_ppCoord[51],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[52] = new Coordinate[2];
  m_ppCoord[52][0].X = 265; m_ppCoord[52][0].Y = 350;
  m_ppCoord[52][1].X = 390; m_ppCoord[52][1].Y = 350;
  m_ppComponent[52] = new Wire(m_ppCoord[52],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,25,21);

  m_ppCoord[53] = new Coordinate[2];
  m_ppCoord[53][0].X = 275; m_ppCoord[53][0].Y = 380;
  m_ppCoord[53][1].X = 390; m_ppCoord[53][1].Y = 380;
  m_ppComponent[53] = new Wire(m_ppCoord[53],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,20,16);

  m_ppCoord[54] = new Coordinate[3];
  m_ppCoord[54][0].X = 275; m_ppCoord[54][0].Y = 380;
  m_ppCoord[54][1].X = 275; m_ppCoord[54][1].Y = 400;
  m_ppCoord[54][2].X = 290; m_ppCoord[54][2].Y = 400;
  m_ppComponent[54] = new Wire(m_ppCoord[54],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,20,16);

  m_ppCoord[55] = new Coordinate[2];
  m_ppCoord[55][0].X = 265; m_ppCoord[55][0].Y = 390;
  m_ppCoord[55][1].X = 265; m_ppCoord[55][1].Y = 440;
  m_ppComponent[55] = new Wire(m_ppCoord[55],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[56] = new Coordinate[2];
  m_ppCoord[56][0].X = 265; m_ppCoord[56][0].Y = 440;
  m_ppCoord[56][1].X = 290; m_ppCoord[56][1].Y = 440;
  m_ppComponent[56] = new Wire(m_ppCoord[56],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,15,11);

  m_ppCoord[57] = new Coordinate[3];
  m_ppCoord[57][0].X = 265; m_ppCoord[57][0].Y = 440;
  m_ppCoord[57][1].X = 265; m_ppCoord[57][1].Y = 555;
  m_ppCoord[57][2].X = 345; m_ppCoord[57][2].Y = 555;
  m_ppComponent[57] = new Wire(m_ppCoord[57],3,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,15,0);

  m_ppCoord[58] = new Coordinate[2];
  m_ppCoord[58][0].X = 315; m_ppCoord[58][0].Y = 420;
  m_ppCoord[58][1].X = 340; m_ppCoord[58][1].Y = 420;
  m_ppComponent[58] = new Wire(m_ppCoord[58],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,4,0);

  m_ppCoord[59] = new Coordinate[2];
  m_ppCoord[59][0].X = 330; m_ppCoord[59][0].Y = 460;
  m_ppCoord[59][1].X = 340; m_ppCoord[59][1].Y = 460;
  m_ppComponent[59] = new Wire(m_ppCoord[59],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,4,0);

  m_ppCoord[60] = new Coordinate[2];
  m_ppCoord[60][0].X = 365; m_ppCoord[60][0].Y = 445;
  m_ppCoord[60][1].X = 390; m_ppCoord[60][1].Y = 445;
  m_ppComponent[60] = new Wire(m_ppCoord[60],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,4,0);

  m_ppCoord[61] = new Coordinate[2];
  m_ppCoord[61][0].X = 365; m_ppCoord[61][0].Y = 505;
  m_ppCoord[61][1].X = 390; m_ppCoord[61][1].Y = 505;
  m_ppComponent[61] = new Wire(m_ppCoord[61],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[62] = new Coordinate[2];
  m_ppCoord[62][0].X = 345; m_ppCoord[62][0].Y = 555;
  m_ppCoord[62][1].X = 380; m_ppCoord[62][1].Y = 555;
  m_ppComponent[62] = new Wire(m_ppCoord[62],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,15,0);

  m_ppCoord[63] = new Coordinate[3];
  m_ppCoord[63][0].X = 345; m_ppCoord[63][0].Y = 555;
  m_ppCoord[63][1].X = 345; m_ppCoord[63][1].Y = 590;
  m_ppCoord[63][2].X = 480; m_ppCoord[63][2].Y = 590;
  m_ppComponent[63] = new Wire(m_ppCoord[63],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[64] = new Coordinate[2];
  m_ppCoord[64][0].X = 464; m_ppCoord[64][0].Y = 555;
  m_ppCoord[64][1].X = 480; m_ppCoord[64][1].Y = 555;
  m_ppComponent[64] = new Wire(m_ppCoord[64],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[65] = new Coordinate[3];
  m_ppCoord[65][0].X = 505; m_ppCoord[65][0].Y = 580;
  m_ppCoord[65][1].X = 540; m_ppCoord[65][1].Y = 580;
  m_ppCoord[65][2].X = 540; m_ppCoord[65][2].Y = 540;
  m_ppComponent[65] = new Wire(m_ppCoord[65],3,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[66] = new Coordinate[2];
  m_ppCoord[66][0].X = 540; m_ppCoord[66][0].Y = 540;
  m_ppCoord[66][1].X = 540; m_ppCoord[66][1].Y = 460;
  m_ppComponent[66] = new Wire(m_ppCoord[66],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[67] = new Coordinate[2];
  m_ppCoord[67][0].X = 540; m_ppCoord[67][0].Y = 540;
  m_ppCoord[67][1].X = 560; m_ppCoord[67][1].Y = 540;
  m_ppComponent[67] = new Wire(m_ppCoord[67],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,10,6);

  m_ppCoord[68] = new Coordinate[3];
  m_ppCoord[68][0].X = 540; m_ppCoord[68][0].Y = 460;
  m_ppCoord[68][1].X = 540; m_ppCoord[68][1].Y = 142;
  m_ppCoord[68][2].X = 620; m_ppCoord[68][2].Y = 142;
  m_ppComponent[68] = new Wire(m_ppCoord[68],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[69] = new Coordinate[2];
  m_ppCoord[69][0].X = 540; m_ppCoord[69][0].Y = 460;
  m_ppCoord[69][1].X = 560; m_ppCoord[69][1].Y = 460;
  m_ppComponent[69] = new Wire(m_ppCoord[69],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[70] = new Coordinate[2];
  m_ppCoord[70][0].X = 657; m_ppCoord[70][0].Y = 142;
  m_ppCoord[70][1].X = 700; m_ppCoord[70][1].Y = 142;
  m_ppComponent[70] = new Wire(m_ppCoord[70],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[71] = new Coordinate[2];
  m_ppCoord[71][0].X = 535; m_ppCoord[71][0].Y = 55;
  m_ppCoord[71][1].X = 880; m_ppCoord[71][1].Y = 55;
  m_ppComponent[71] = new Wire(m_ppCoord[71],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[72] = new Coordinate[2];
  m_ppCoord[72][0].X = 761; m_ppCoord[72][0].Y = 110;
  m_ppCoord[72][1].X = 830; m_ppCoord[72][1].Y = 110;
  m_ppComponent[72] = new Wire(m_ppCoord[72],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[73] = new Coordinate[2];
  m_ppCoord[73][0].X = 855; m_ppCoord[73][0].Y = 90;
  m_ppCoord[73][1].X = 880; m_ppCoord[73][1].Y = 90;
  m_ppComponent[73] = new Wire(m_ppCoord[73],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[74] = new Coordinate[6];
  m_ppCoord[74][0].X = 905; m_ppCoord[74][0].Y = 70;
  m_ppCoord[74][1].X = 920; m_ppCoord[74][1].Y = 70;
  m_ppCoord[74][2].X = 920; m_ppCoord[74][2].Y = 5;
  m_ppCoord[74][3].X = 20; m_ppCoord[74][3].Y = 5;
  m_ppCoord[74][4].X = 20; m_ppCoord[74][4].Y = 360;
  m_ppCoord[74][5].X = 40; m_ppCoord[74][5].Y = 360;
  m_ppComponent[74] = new Wire(m_ppCoord[74],6,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[75] = new Coordinate[2];
  m_ppCoord[75][0].X = 475; m_ppCoord[75][0].Y = 365;
  m_ppCoord[75][1].X = 500; m_ppCoord[75][1].Y = 365;
  m_ppComponent[75] = new Wire(m_ppCoord[75],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[76] = new Coordinate[2];
  m_ppCoord[76][0].X = 500; m_ppCoord[76][0].Y = 365;
  m_ppCoord[76][1].X = 500; m_ppCoord[76][1].Y = 330;
  m_ppComponent[76] = new Wire(m_ppCoord[76],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[77] = new Coordinate[3];
  m_ppCoord[77][0].X = 500; m_ppCoord[77][0].Y = 330;
  m_ppCoord[77][1].X = 500; m_ppCoord[77][1].Y = 70;
  m_ppCoord[77][2].X = 510; m_ppCoord[77][2].Y = 70;
  m_ppComponent[77] = new Wire(m_ppCoord[77],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[78] = new Coordinate[2];
  m_ppCoord[78][0].X = 500; m_ppCoord[78][0].Y = 330;
  m_ppCoord[78][1].X = 610; m_ppCoord[78][1].Y = 330;
  m_ppComponent[78] = new Wire(m_ppCoord[78],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[79] = new Coordinate[2];
  m_ppCoord[79][0].X = 590; m_ppCoord[79][0].Y = 300;
  m_ppCoord[79][1].X = 610; m_ppCoord[79][1].Y = 300;
  m_ppComponent[79] = new Wire(m_ppCoord[79],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[80] = new Coordinate[2];
  m_ppCoord[80][0].X = 500; m_ppCoord[80][0].Y = 365;
  m_ppCoord[80][1].X = 620; m_ppCoord[80][1].Y = 365;
  m_ppComponent[80] = new Wire(m_ppCoord[80],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[81] = new Coordinate[3];
  m_ppCoord[81][0].X = 500; m_ppCoord[81][0].Y = 365;
  m_ppCoord[81][1].X = 500; m_ppCoord[81][1].Y = 500;
  m_ppCoord[81][2].X = 560; m_ppCoord[81][2].Y = 500;
  m_ppComponent[81] = new Wire(m_ppCoord[81],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,4,0);

  m_ppCoord[82] = new Coordinate[2];
  m_ppCoord[82][0].X = 475; m_ppCoord[82][0].Y = 425;
  m_ppCoord[82][1].X = 520; m_ppCoord[82][1].Y = 425;
  m_ppComponent[82] = new Wire(m_ppCoord[82],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[83] = new Coordinate[2];
  m_ppCoord[83][0].X = 520; m_ppCoord[83][0].Y = 425;
  m_ppCoord[83][1].X = 560; m_ppCoord[83][1].Y = 425;
  m_ppComponent[83] = new Wire(m_ppCoord[83],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[84] = new Coordinate[2];
  m_ppCoord[84][0].X = 520; m_ppCoord[84][0].Y = 425;
  m_ppCoord[84][1].X = 520; m_ppCoord[84][1].Y = 475;
  m_ppComponent[84] = new Wire(m_ppCoord[84],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[85] = new Coordinate[2];
  m_ppCoord[85][0].X = 520; m_ppCoord[85][0].Y = 475;
  m_ppCoord[85][1].X = 820; m_ppCoord[85][1].Y = 475;
  m_ppComponent[85] = new Wire(m_ppCoord[85],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[86] = new Coordinate[3];
  m_ppCoord[86][0].X = 520; m_ppCoord[86][0].Y = 475;
  m_ppCoord[86][1].X = 520; m_ppCoord[86][1].Y = 555;
  m_ppCoord[86][2].X = 620; m_ppCoord[86][2].Y = 555;
  m_ppComponent[86] = new Wire(m_ppCoord[86],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[87] = new Coordinate[4];
  m_ppCoord[87][0].X = 585; m_ppCoord[87][0].Y = 520;
  m_ppCoord[87][1].X = 592; m_ppCoord[87][1].Y = 520;
  m_ppCoord[87][2].X = 592; m_ppCoord[87][2].Y = 540;
  m_ppCoord[87][3].X = 620; m_ppCoord[87][3].Y = 540;
  m_ppComponent[87] = new Wire(m_ppCoord[87],4,eDisplayMode,WIRE_ARROW,WIRE_BLACK,4,0);

  m_ppCoord[88] = new Coordinate[2];
  m_ppCoord[88][0].X = 585; m_ppCoord[88][0].Y = 435;
  m_ppCoord[88][1].X = 620; m_ppCoord[88][1].Y = 435;
  m_ppComponent[88] = new Wire(m_ppCoord[88],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[89] = new Coordinate[2];
  m_ppCoord[89][0].X = 680; m_ppCoord[89][0].Y = 410;
  m_ppCoord[89][1].X = 795; m_ppCoord[89][1].Y = 410;
  m_ppComponent[89] = new Wire(m_ppCoord[89],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,31,0);

  m_ppCoord[90] = new Coordinate[2];
  m_ppCoord[90][0].X = 795; m_ppCoord[90][0].Y = 410;
  m_ppCoord[90][1].X = 820; m_ppCoord[90][1].Y = 410;
  m_ppComponent[90] = new Wire(m_ppCoord[90],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[91] = new Coordinate[3];
  m_ppCoord[91][0].X = 795; m_ppCoord[91][0].Y = 410;
  m_ppCoord[91][1].X = 795; m_ppCoord[91][1].Y = 530;
  m_ppCoord[91][2].X = 950; m_ppCoord[91][2].Y = 530;
  m_ppComponent[91] = new Wire(m_ppCoord[91],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[92] = new Coordinate[2];
  m_ppCoord[92][0].X = 929; m_ppCoord[92][0].Y = 490;
  m_ppCoord[92][1].X = 950; m_ppCoord[92][1].Y = 490;
  m_ppComponent[92] = new Wire(m_ppCoord[92],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[93] = new Coordinate[4];
  m_ppCoord[93][0].X = 975; m_ppCoord[93][0].Y = 510;
  m_ppCoord[93][1].X = 990; m_ppCoord[93][1].Y = 510;
  m_ppCoord[93][2].X = 990; m_ppCoord[93][2].Y = 625;
  m_ppCoord[93][3].X = 615; m_ppCoord[93][3].Y = 625;
  m_ppComponent[93] = new Wire(m_ppCoord[93],4,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[94] = new Coordinate[4];
  m_ppCoord[94][0].X = 680; m_ppCoord[94][0].Y = 555;
  m_ppCoord[94][1].X = 700; m_ppCoord[94][1].Y = 555;
  m_ppCoord[94][2].X = 700; m_ppCoord[94][2].Y = 590;
  m_ppCoord[94][3].X = 615; m_ppCoord[94][3].Y = 590;
  m_ppComponent[94] = new Wire(m_ppCoord[94],4,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[95] = new Coordinate[4];
  m_ppCoord[95][0].X = 590; m_ppCoord[95][0].Y = 610;
  m_ppCoord[95][1].X = 320; m_ppCoord[95][1].Y = 610;
  m_ppCoord[95][2].X = 320; m_ppCoord[95][2].Y = 540;
  m_ppCoord[95][3].X = 340; m_ppCoord[95][3].Y = 540;
  m_ppComponent[95] = new Wire(m_ppCoord[95],4,eDisplayMode,WIRE_ARROW,WIRE_BLACK,31,0);

  m_ppCoord[96] = new Coordinate[2];
  m_ppCoord[96][0].X = 680; m_ppCoord[96][0].Y = 380;
  m_ppCoord[96][1].X = 690; m_ppCoord[96][1].Y = 380;
  m_ppComponent[96] = new Wire(m_ppCoord[96],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,0,0);

  m_ppCoord[97] = new Coordinate[2];
  m_ppCoord[97][0].X = 690; m_ppCoord[97][0].Y = 380;
  m_ppCoord[97][1].X = 750; m_ppCoord[97][1].Y = 380;
  m_ppComponent[97] = new Wire(m_ppCoord[97],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[98] = new Coordinate[3];
  m_ppCoord[98][0].X = 690; m_ppCoord[98][0].Y = 380;
  m_ppCoord[98][1].X = 690; m_ppCoord[98][1].Y = 342;
  m_ppCoord[98][2].X = 705; m_ppCoord[98][2].Y = 342;
  m_ppComponent[98] = new Wire(m_ppCoord[98],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[99] = new Coordinate[2];
  m_ppCoord[99][0].X = 730; m_ppCoord[99][0].Y = 342;
  m_ppCoord[99][1].X = 750; m_ppCoord[99][1].Y = 342;
  m_ppComponent[99] = new Wire(m_ppCoord[99],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[100] = new Coordinate[2];
  m_ppCoord[100][0].X = 659; m_ppCoord[100][0].Y = 315;
  m_ppCoord[100][1].X = 810; m_ppCoord[100][1].Y = 315;
  m_ppComponent[100] = new Wire(m_ppCoord[100],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[101] = new Coordinate[2];
  m_ppCoord[101][0].X = 775; m_ppCoord[101][0].Y = 350;
  m_ppCoord[101][1].X = 795; m_ppCoord[101][1].Y = 350;
  m_ppComponent[101] = new Wire(m_ppCoord[101],2,eDisplayMode,WIRE_CIRCLE,WIRE_BLACK,0,0);

  m_ppCoord[102] = new Coordinate[2];
  m_ppCoord[102][0].X = 795; m_ppCoord[102][0].Y = 350;
  m_ppCoord[102][1].X = 850; m_ppCoord[102][1].Y = 350;
  m_ppComponent[102] = new Wire(m_ppCoord[102],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[103] = new Coordinate[3];
  m_ppCoord[103][0].X = 795; m_ppCoord[103][0].Y = 350;
  m_ppCoord[103][1].X = 795; m_ppCoord[103][1].Y = 325;
  m_ppCoord[103][2].X = 810; m_ppCoord[103][2].Y = 325;
  m_ppComponent[103] = new Wire(m_ppCoord[103],3,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[104] = new Coordinate[2];
  m_ppCoord[104][0].X = 835; m_ppCoord[104][0].Y = 310;
  m_ppCoord[104][1].X = 850; m_ppCoord[104][1].Y = 310;
  m_ppComponent[104] = new Wire(m_ppCoord[104],2,eDisplayMode,WIRE_ARROW,WIRE_BLACK,0,0);

  m_ppCoord[105] = new Coordinate[3];
  m_ppCoord[105][0].X = 275; m_ppCoord[105][0].Y = 140;
  m_ppCoord[105][1].X = 520; m_ppCoord[105][1].Y = 140;
  m_ppCoord[105][2].X = 520; m_ppCoord[105][2].Y = 80;
  m_ppComponent[105] = new Wire(m_ppCoord[105],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[106] = new Coordinate[5];
  m_ppCoord[106][0].X = 280; m_ppCoord[106][0].Y = 152;
  m_ppCoord[106][1].X = 550; m_ppCoord[106][1].Y = 152;
  m_ppCoord[106][2].X = 550; m_ppCoord[106][2].Y = 20;
  m_ppCoord[106][3].X = 890; m_ppCoord[106][3].Y = 20;
  m_ppCoord[106][4].X = 890; m_ppCoord[106][4].Y = 40;
  m_ppComponent[106] = new Wire(m_ppCoord[106],5,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[107] = new Coordinate[3];
  m_ppCoord[107][0].X = 283; m_ppCoord[107][0].Y = 164;
  m_ppCoord[107][1].X = 830; m_ppCoord[107][1].Y = 164;
  m_ppCoord[107][2].X = 830; m_ppCoord[107][2].Y = 156;
  m_ppComponent[107] = new Wire(m_ppCoord[107],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[108] = new Coordinate[3];
  m_ppCoord[108][0].X = 285; m_ppCoord[108][0].Y = 176;
  m_ppCoord[108][1].X = 960; m_ppCoord[108][1].Y = 176;
  m_ppCoord[108][2].X = 960; m_ppCoord[108][2].Y = 480;
  m_ppComponent[108] = new Wire(m_ppCoord[108],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[109] = new Coordinate[3];
  m_ppCoord[109][0].X = 287; m_ppCoord[109][0].Y = 188;
  m_ppCoord[109][1].X = 920; m_ppCoord[109][1].Y = 188;
  m_ppCoord[109][2].X = 920; m_ppCoord[109][2].Y = 400;
  m_ppComponent[109] = new Wire(m_ppCoord[109],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[110] = new Coordinate[3];
  m_ppCoord[110][0].X = 288; m_ppCoord[110][0].Y = 200;
  m_ppCoord[110][1].X = 905; m_ppCoord[110][1].Y = 200;
  m_ppCoord[110][2].X = 905; m_ppCoord[110][2].Y = 400;
  m_ppComponent[110] = new Wire(m_ppCoord[110],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[111] = new Coordinate[3];
  m_ppCoord[111][0].X = 289; m_ppCoord[111][0].Y = 212;
  m_ppCoord[111][1].X = 860; m_ppCoord[111][1].Y = 212;
  m_ppCoord[111][2].X = 860; m_ppCoord[111][2].Y = 300;
  m_ppComponent[111] = new Wire(m_ppCoord[111],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[112] = new Coordinate[3];
  m_ppCoord[112][0].X = 289; m_ppCoord[112][0].Y = 224;
  m_ppCoord[112][1].X = 760; m_ppCoord[112][1].Y = 224;
  m_ppCoord[112][2].X = 760; m_ppCoord[112][2].Y = 330;
  m_ppComponent[112] = new Wire(m_ppCoord[112],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[113] = new Coordinate[3];
  m_ppCoord[113][0].X = 289; m_ppCoord[113][0].Y = 236;
  m_ppCoord[113][1].X = 670; m_ppCoord[113][1].Y = 236;
  m_ppCoord[113][2].X = 670; m_ppCoord[113][2].Y = 366;
  m_ppComponent[113] = new Wire(m_ppCoord[113],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,3,0);

  m_ppCoord[114] = new Coordinate[3];
  m_ppCoord[114][0].X = 289; m_ppCoord[114][0].Y = 248;
  m_ppCoord[114][1].X = 600; m_ppCoord[114][1].Y = 248;
  m_ppCoord[114][2].X = 600; m_ppCoord[114][2].Y = 578;
  m_ppComponent[114] = new Wire(m_ppCoord[114],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[115] = new Coordinate[3];
  m_ppCoord[115][0].X = 288; m_ppCoord[115][0].Y = 260;
  m_ppCoord[115][1].X = 570; m_ppCoord[115][1].Y = 260;
  m_ppCoord[115][2].X = 570; m_ppCoord[115][2].Y = 400;
  m_ppComponent[115] = new Wire(m_ppCoord[115],3,eDisplayMode,WIRE_CIRCLE,WIRE_GREEN,0,0);

  m_ppCoord[116] = new Coordinate[4];
  m_ppCoord[116][0].X = 287; m_ppCoord[116][0].Y = 272;
  m_ppCoord[116][1].X = 510; m_ppCoord[116][1].Y = 272;
  m_ppCoord[116][2].X = 510; m_ppCoord[116][2].Y = 570;
  m_ppCoord[116][3].X = 620; m_ppCoord[116][3].Y = 570;
  m_ppComponent[116] = new Wire(m_ppCoord[116],4,eDisplayMode,WIRE_ARROW,WIRE_GREEN,2,0);

  m_ppCoord[117] = new Coordinate[3];
  m_ppCoord[117][0].X = 285; m_ppCoord[117][0].Y = 284;
  m_ppCoord[117][1].X = 490; m_ppCoord[117][1].Y = 284;
  m_ppCoord[117][2].X = 490; m_ppCoord[117][2].Y = 540;
  m_ppComponent[117] = new Wire(m_ppCoord[117],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[118] = new Coordinate[3];
  m_ppCoord[118][0].X = 283; m_ppCoord[118][0].Y = 296;
  m_ppCoord[118][1].X = 430; m_ppCoord[118][1].Y = 296;
  m_ppCoord[118][2].X = 430; m_ppCoord[118][2].Y = 330;
  m_ppComponent[118] = new Wire(m_ppCoord[118],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[119] = new Coordinate[3];
  m_ppCoord[119][0].X = 280; m_ppCoord[119][0].Y = 308;
  m_ppCoord[119][1].X = 350; m_ppCoord[119][1].Y = 308;
  m_ppCoord[119][2].X = 350; m_ppCoord[119][2].Y = 400;
  m_ppComponent[119] = new Wire(m_ppCoord[119],3,eDisplayMode,WIRE_CIRCLE,WIRE_GREEN,0,0);

  m_ppCoord[120] = new Coordinate[3];
  m_ppCoord[120][0].X = 275; m_ppCoord[120][0].Y = 320;
  m_ppCoord[120][1].X = 300; m_ppCoord[120][1].Y = 320;
  m_ppCoord[120][2].X = 300; m_ppCoord[120][2].Y = 390;
  m_ppComponent[120] = new Wire(m_ppCoord[120],3,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[121] = new Coordinate[5];
  m_ppCoord[121][0].X = 350; m_ppCoord[121][0].Y = 400;
  m_ppCoord[121][1].X = 375; m_ppCoord[121][1].Y = 400;
  m_ppCoord[121][2].X = 375; m_ppCoord[121][2].Y = 480;
  m_ppCoord[121][3].X = 350; m_ppCoord[121][3].Y = 480;
  m_ppCoord[121][4].X = 350; m_ppCoord[121][4].Y = 490;
  m_ppComponent[121] = new Wire(m_ppCoord[121],5,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[122] = new Coordinate[2];
  m_ppCoord[122][0].X = 350; m_ppCoord[122][0].Y = 400;
  m_ppCoord[122][1].X = 350; m_ppCoord[122][1].Y = 410;
  m_ppComponent[122] = new Wire(m_ppCoord[122],2,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[123] = new Coordinate[5];
  m_ppCoord[123][0].X = 570; m_ppCoord[123][0].Y = 400;
  m_ppCoord[123][1].X = 592; m_ppCoord[123][1].Y = 400;
  m_ppCoord[123][2].X = 592; m_ppCoord[123][2].Y = 480;
  m_ppCoord[123][3].X = 570; m_ppCoord[123][3].Y = 480;
  m_ppCoord[123][4].X = 570; m_ppCoord[123][4].Y = 490;
  m_ppComponent[123] = new Wire(m_ppCoord[123],5,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[124] = new Coordinate[2];
  m_ppCoord[124][0].X = 570; m_ppCoord[124][0].Y = 400;
  m_ppCoord[124][1].X = 570; m_ppCoord[124][1].Y = 410;
  m_ppComponent[124] = new Wire(m_ppCoord[124],2,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[125] = new Coordinate[5];
  m_ppCoord[125][0].X = 875; m_ppCoord[125][0].Y = 325;
  m_ppCoord[125][1].X = 885; m_ppCoord[125][1].Y = 325;
  m_ppCoord[125][2].X = 885; m_ppCoord[125][2].Y = 164;
  m_ppCoord[125][3].X = 860; m_ppCoord[125][3].Y = 164;
  m_ppCoord[125][4].X = 860; m_ppCoord[125][4].Y = 156;
  m_ppComponent[125] = new Wire(m_ppCoord[125],5,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  m_ppCoord[126] = new Coordinate[2];
  m_ppCoord[126][0].X = 842; m_ppCoord[126][0].Y = 130;
  m_ppCoord[126][1].X = 842; m_ppCoord[126][1].Y = 120;
  m_ppComponent[126] = new Wire(m_ppCoord[126],2,eDisplayMode,WIRE_ARROW,WIRE_GREEN,0,0);

  // Start linking components

  m_ppComponent[0]->DependsOn(1);
  m_ppComponent[0]->DependsOn(m_ppComponent[74],0);
  
  m_ppComponent[1]->DependsOn(1);
  m_ppComponent[1]->DependsOn(m_ppComponent[50],0);

  m_ppComponent[2]->DependsOn(4);
  m_ppComponent[2]->DependsOn(m_ppComponent[90],0);
  m_ppComponent[2]->DependsOn(m_ppComponent[85],0);
  m_ppComponent[2]->DependsOn(m_ppComponent[109],0);
  m_ppComponent[2]->DependsOn(m_ppComponent[110],0);

  m_ppComponent[3]->DependsOn(1);
  m_ppComponent[3]->DependsOn(m_ppComponent[33],0);

  m_ppComponent[4]->DependsOn(5);
  m_ppComponent[4]->DependsOn(m_ppComponent[52],0);
  m_ppComponent[4]->DependsOn(m_ppComponent[53],0);
  m_ppComponent[4]->DependsOn(m_ppComponent[60],0);
  m_ppComponent[4]->DependsOn(m_ppComponent[61],0);
  m_ppComponent[4]->DependsOn(m_ppComponent[118],0);

  m_ppComponent[5]->DependsOn(3);
  m_ppComponent[5]->DependsOn(m_ppComponent[54],0);
  m_ppComponent[5]->DependsOn(m_ppComponent[56],0);
  m_ppComponent[5]->DependsOn(m_ppComponent[120],0);

  m_ppComponent[6]->DependsOn(3);
  m_ppComponent[6]->DependsOn(m_ppComponent[58],0);
  m_ppComponent[6]->DependsOn(m_ppComponent[59],0);
  m_ppComponent[6]->DependsOn(m_ppComponent[122],0);

  m_ppComponent[7]->DependsOn(3);
  m_ppComponent[7]->DependsOn(m_ppComponent[95],0);
  m_ppComponent[7]->DependsOn(m_ppComponent[40],0);
  m_ppComponent[7]->DependsOn(m_ppComponent[121],0);

  m_ppComponent[8]->DependsOn(3);
  m_ppComponent[8]->DependsOn(m_ppComponent[64],0);
  m_ppComponent[8]->DependsOn(m_ppComponent[63],0);
  m_ppComponent[8]->DependsOn(m_ppComponent[117],0);

  m_ppComponent[9]->DependsOn(3);
  m_ppComponent[9]->DependsOn(m_ppComponent[45],0);
  m_ppComponent[9]->DependsOn(m_ppComponent[77],0);
  m_ppComponent[9]->DependsOn(m_ppComponent[105],0);

  m_ppComponent[10]->DependsOn(3);
  m_ppComponent[10]->DependsOn(m_ppComponent[83],0);
  m_ppComponent[10]->DependsOn(m_ppComponent[69],0);
  m_ppComponent[10]->DependsOn(m_ppComponent[124],0);

  m_ppComponent[11]->DependsOn(3);
  m_ppComponent[11]->DependsOn(m_ppComponent[81],0);
  m_ppComponent[11]->DependsOn(m_ppComponent[67],0);
  m_ppComponent[11]->DependsOn(m_ppComponent[123],0);

  m_ppComponent[12]->DependsOn(3);
  m_ppComponent[12]->DependsOn(m_ppComponent[93],0);
  m_ppComponent[12]->DependsOn(m_ppComponent[94],0);
  m_ppComponent[12]->DependsOn(m_ppComponent[114],0);

  m_ppComponent[13]->DependsOn(3);
  m_ppComponent[13]->DependsOn(m_ppComponent[97],0);
  m_ppComponent[13]->DependsOn(m_ppComponent[99],0);
  m_ppComponent[13]->DependsOn(m_ppComponent[112],0);

  m_ppComponent[14]->DependsOn(3);
  m_ppComponent[14]->DependsOn(m_ppComponent[102],0);
  m_ppComponent[14]->DependsOn(m_ppComponent[104],0);
  m_ppComponent[14]->DependsOn(m_ppComponent[111],0);

  m_ppComponent[15]->DependsOn(3);
  m_ppComponent[15]->DependsOn(m_ppComponent[47],0);
  m_ppComponent[15]->DependsOn(m_ppComponent[72],0);
  m_ppComponent[15]->DependsOn(m_ppComponent[126],0);

  m_ppComponent[16]->DependsOn(3);
  m_ppComponent[16]->DependsOn(m_ppComponent[73],0);
  m_ppComponent[16]->DependsOn(m_ppComponent[71],0);
  m_ppComponent[16]->DependsOn(m_ppComponent[106],0);

  m_ppComponent[17]->DependsOn(3);
  m_ppComponent[17]->DependsOn(m_ppComponent[91],0);
  m_ppComponent[17]->DependsOn(m_ppComponent[92],0);
  m_ppComponent[17]->DependsOn(m_ppComponent[108],0);

  m_ppComponent[18]->DependsOn(0);

  m_ppComponent[19]->DependsOn(0);

  m_ppComponent[20]->DependsOn(0);

  m_ppComponent[21]->DependsOn(3);
  m_ppComponent[21]->DependsOn(m_ppComponent[80],0);
  m_ppComponent[21]->DependsOn(m_ppComponent[88],0);
  m_ppComponent[21]->DependsOn(m_ppComponent[113],0);

  m_ppComponent[22]->DependsOn(3);
  m_ppComponent[22]->DependsOn(m_ppComponent[86],0);
  m_ppComponent[22]->DependsOn(m_ppComponent[87],0);
  m_ppComponent[22]->DependsOn(m_ppComponent[116],0);

  m_ppComponent[23]->DependsOn(1);
  m_ppComponent[23]->DependsOn(m_ppComponent[62],0);

  m_ppComponent[24]->DependsOn(2);
  m_ppComponent[24]->DependsOn(m_ppComponent[34],0);
  m_ppComponent[24]->DependsOn(m_ppComponent[35],0);

  m_ppComponent[25]->DependsOn(2);
  m_ppComponent[25]->DependsOn(m_ppComponent[46],0);
  m_ppComponent[25]->DependsOn(m_ppComponent[70],0);

  m_ppComponent[26]->DependsOn(1);
  m_ppComponent[26]->DependsOn(m_ppComponent[39],0);

  m_ppComponent[27]->DependsOn(1);
  m_ppComponent[27]->DependsOn(m_ppComponent[68],0);

  m_ppComponent[28]->DependsOn(2);
  m_ppComponent[28]->DependsOn(m_ppComponent[107],0);
  m_ppComponent[28]->DependsOn(m_ppComponent[125],0);

  m_ppComponent[29]->DependsOn(2);
  m_ppComponent[29]->DependsOn(m_ppComponent[100],0);
  m_ppComponent[29]->DependsOn(m_ppComponent[103],0);

  m_ppComponent[30]->DependsOn(1);
  m_ppComponent[30]->DependsOn(m_ppComponent[98],0);

  m_ppComponent[31]->DependsOn(2);
  m_ppComponent[31]->DependsOn(m_ppComponent[78],0);
  m_ppComponent[31]->DependsOn(m_ppComponent[79],0);

  m_ppComponent[32]->DependsOn(1);
  m_ppComponent[32]->DependsOn(m_ppComponent[0],0);

  m_ppComponent[33]->DependsOn(1);
  m_ppComponent[33]->DependsOn(m_ppComponent[32],0);

  m_ppComponent[34]->DependsOn(1);
  m_ppComponent[34]->DependsOn(m_ppComponent[32],0);

  m_ppComponent[35]->DependsOn(1);
  m_ppComponent[35]->DependsOn(m_ppComponent[18],0);

  m_ppComponent[36]->DependsOn(1);
  m_ppComponent[36]->DependsOn(m_ppComponent[24],0);

  m_ppComponent[37]->DependsOn(1);
  m_ppComponent[37]->DependsOn(m_ppComponent[3],0);

  m_ppComponent[38]->DependsOn(1);
  m_ppComponent[38]->DependsOn(m_ppComponent[36],0);

  m_ppComponent[39]->DependsOn(1);
  m_ppComponent[39]->DependsOn(m_ppComponent[37],0);

  m_ppComponent[40]->DependsOn(1);
  m_ppComponent[40]->DependsOn(m_ppComponent[36],0);

  m_ppComponent[41]->DependsOn(1);
  m_ppComponent[41]->DependsOn(m_ppComponent[37],0);

  m_ppComponent[42]->DependsOn(1);
  m_ppComponent[42]->DependsOn(m_ppComponent[38],0);

  m_ppComponent[43]->DependsOn(1);
  m_ppComponent[43]->DependsOn(m_ppComponent[26],0);

  m_ppComponent[44]->DependsOn(1);
  m_ppComponent[44]->DependsOn(m_ppComponent[38],0);

  m_ppComponent[45]->DependsOn(2);
  m_ppComponent[45]->DependsOn(m_ppComponent[42],1);
  m_ppComponent[45]->DependsOn(m_ppComponent[43],1);

  m_ppComponent[46]->DependsOn(1);
  m_ppComponent[46]->DependsOn(m_ppComponent[44],0);

  m_ppComponent[47]->DependsOn(1);
  m_ppComponent[47]->DependsOn(m_ppComponent[44],0);

  m_ppComponent[48]->DependsOn(1);
  m_ppComponent[48]->DependsOn(m_ppComponent[41],0);

  m_ppComponent[49]->DependsOn(1);
  m_ppComponent[49]->DependsOn(m_ppComponent[48],0);

  m_ppComponent[50]->DependsOn(1);
  m_ppComponent[50]->DependsOn(m_ppComponent[49],0);

  m_ppComponent[51]->DependsOn(1);
  m_ppComponent[51]->DependsOn(m_ppComponent[48],0);

  m_ppComponent[52]->DependsOn(1);
  m_ppComponent[52]->DependsOn(m_ppComponent[49],0);

  m_ppComponent[53]->DependsOn(1);
  m_ppComponent[53]->DependsOn(m_ppComponent[51],0);

  m_ppComponent[54]->DependsOn(1);
  m_ppComponent[54]->DependsOn(m_ppComponent[51],0);

  m_ppComponent[55]->DependsOn(1);
  m_ppComponent[55]->DependsOn(m_ppComponent[41],0);

  m_ppComponent[56]->DependsOn(1);
  m_ppComponent[56]->DependsOn(m_ppComponent[55],0);

  m_ppComponent[57]->DependsOn(1);
  m_ppComponent[57]->DependsOn(m_ppComponent[55],0);

  m_ppComponent[58]->DependsOn(1);
  m_ppComponent[58]->DependsOn(m_ppComponent[5],0);

  m_ppComponent[59]->DependsOn(1);
  m_ppComponent[59]->DependsOn(m_ppComponent[19],0);

  m_ppComponent[60]->DependsOn(1);
  m_ppComponent[60]->DependsOn(m_ppComponent[6],0);

  m_ppComponent[61]->DependsOn(1);
  m_ppComponent[61]->DependsOn(m_ppComponent[7],0);

  m_ppComponent[62]->DependsOn(1);
  m_ppComponent[62]->DependsOn(m_ppComponent[57],0);

  m_ppComponent[63]->DependsOn(1);
  m_ppComponent[63]->DependsOn(m_ppComponent[57],0);

  m_ppComponent[64]->DependsOn(1);
  m_ppComponent[64]->DependsOn(m_ppComponent[23],0);

  m_ppComponent[65]->DependsOn(1);
  m_ppComponent[65]->DependsOn(m_ppComponent[8],0);

  m_ppComponent[66]->DependsOn(1);
  m_ppComponent[66]->DependsOn(m_ppComponent[65],0);

  m_ppComponent[67]->DependsOn(1);
  m_ppComponent[67]->DependsOn(m_ppComponent[65],0);

  m_ppComponent[68]->DependsOn(1);
  m_ppComponent[68]->DependsOn(m_ppComponent[66],0);

  m_ppComponent[69]->DependsOn(1);
  m_ppComponent[69]->DependsOn(m_ppComponent[66],0);

  m_ppComponent[70]->DependsOn(1);
  m_ppComponent[70]->DependsOn(m_ppComponent[27],0);

  m_ppComponent[71]->DependsOn(1);
  m_ppComponent[71]->DependsOn(m_ppComponent[9],0);

  m_ppComponent[72]->DependsOn(1);
  m_ppComponent[72]->DependsOn(m_ppComponent[25],0);

  m_ppComponent[73]->DependsOn(1);
  m_ppComponent[73]->DependsOn(m_ppComponent[15],0);

  m_ppComponent[74]->DependsOn(1);
  m_ppComponent[74]->DependsOn(m_ppComponent[16],0);

  m_ppComponent[75]->DependsOn(1);
  m_ppComponent[75]->DependsOn(m_ppComponent[4],0);

  m_ppComponent[76]->DependsOn(1);
  m_ppComponent[76]->DependsOn(m_ppComponent[75],0);

  m_ppComponent[77]->DependsOn(1);
  m_ppComponent[77]->DependsOn(m_ppComponent[76],0);

  m_ppComponent[78]->DependsOn(1);
  m_ppComponent[78]->DependsOn(m_ppComponent[76],0);

  m_ppComponent[79]->DependsOn(1);
  m_ppComponent[79]->DependsOn(m_ppComponent[20],0);

  m_ppComponent[80]->DependsOn(1);
  m_ppComponent[80]->DependsOn(m_ppComponent[75],0);

  m_ppComponent[81]->DependsOn(1);
  m_ppComponent[81]->DependsOn(m_ppComponent[75],0);

  m_ppComponent[82]->DependsOn(1);
  m_ppComponent[82]->DependsOn(m_ppComponent[4],1);

  m_ppComponent[83]->DependsOn(1);
  m_ppComponent[83]->DependsOn(m_ppComponent[82],0);

  m_ppComponent[84]->DependsOn(1);
  m_ppComponent[84]->DependsOn(m_ppComponent[82],0);

  m_ppComponent[85]->DependsOn(1);
  m_ppComponent[85]->DependsOn(m_ppComponent[84],0);

  m_ppComponent[86]->DependsOn(1);
  m_ppComponent[86]->DependsOn(m_ppComponent[84],0);

  m_ppComponent[87]->DependsOn(1);
  m_ppComponent[87]->DependsOn(m_ppComponent[11],0);

  m_ppComponent[88]->DependsOn(1);
  m_ppComponent[88]->DependsOn(m_ppComponent[10],0);

  m_ppComponent[89]->DependsOn(1);
  m_ppComponent[89]->DependsOn(m_ppComponent[21],0);

  m_ppComponent[90]->DependsOn(1);
  m_ppComponent[90]->DependsOn(m_ppComponent[89],0);

  m_ppComponent[91]->DependsOn(1);
  m_ppComponent[91]->DependsOn(m_ppComponent[89],0);

  m_ppComponent[92]->DependsOn(1);
  m_ppComponent[92]->DependsOn(m_ppComponent[2],0);

  m_ppComponent[93]->DependsOn(1);
  m_ppComponent[93]->DependsOn(m_ppComponent[17],0);

  m_ppComponent[94]->DependsOn(1);
  m_ppComponent[94]->DependsOn(m_ppComponent[22],0);

  m_ppComponent[95]->DependsOn(1);
  m_ppComponent[95]->DependsOn(m_ppComponent[12],0);

  m_ppComponent[96]->DependsOn(1);
  m_ppComponent[96]->DependsOn(m_ppComponent[21],1);

  m_ppComponent[97]->DependsOn(1);
  m_ppComponent[97]->DependsOn(m_ppComponent[96],0);

  m_ppComponent[98]->DependsOn(1);
  m_ppComponent[98]->DependsOn(m_ppComponent[96],0);

  m_ppComponent[99]->DependsOn(1);
  m_ppComponent[99]->DependsOn(m_ppComponent[30],0);

  m_ppComponent[100]->DependsOn(1);
  m_ppComponent[100]->DependsOn(m_ppComponent[31],0);

  m_ppComponent[101]->DependsOn(1);
  m_ppComponent[101]->DependsOn(m_ppComponent[13],0);

  m_ppComponent[102]->DependsOn(1);
  m_ppComponent[102]->DependsOn(m_ppComponent[101],0);

  m_ppComponent[103]->DependsOn(1);
  m_ppComponent[103]->DependsOn(m_ppComponent[101],0);

  m_ppComponent[104]->DependsOn(1);
  m_ppComponent[104]->DependsOn(m_ppComponent[29],0);

  m_ppComponent[105]->DependsOn(1);
  m_ppComponent[105]->DependsOn(m_ppComponent[1],_JumpSrc_);

  m_ppComponent[106]->DependsOn(1);
  m_ppComponent[106]->DependsOn(m_ppComponent[1],_Jump_);

  m_ppComponent[107]->DependsOn(1);
  m_ppComponent[107]->DependsOn(m_ppComponent[1],_Branch_);

  m_ppComponent[108]->DependsOn(1);
  m_ppComponent[108]->DependsOn(m_ppComponent[1],_MemToReg_);

  m_ppComponent[109]->DependsOn(1);
  m_ppComponent[109]->DependsOn(m_ppComponent[1],_MemRead_);

  m_ppComponent[110]->DependsOn(1);
  m_ppComponent[110]->DependsOn(m_ppComponent[1],_MemWrite_);

  m_ppComponent[111]->DependsOn(1);
  m_ppComponent[111]->DependsOn(m_ppComponent[1],_ZeroCmp_);

  m_ppComponent[112]->DependsOn(1);
  m_ppComponent[112]->DependsOn(m_ppComponent[1],_ZeroInv_);

  m_ppComponent[113]->DependsOn(1);
  m_ppComponent[113]->DependsOn(m_ppComponent[1],_ALUOp_);

  m_ppComponent[114]->DependsOn(1);
  m_ppComponent[114]->DependsOn(m_ppComponent[1],_DataSrc_);

  m_ppComponent[115]->DependsOn(1);
  m_ppComponent[115]->DependsOn(m_ppComponent[1],_ReadSrc_);

  m_ppComponent[116]->DependsOn(1);
  m_ppComponent[116]->DependsOn(m_ppComponent[1],_ShiftOp_);

  m_ppComponent[117]->DependsOn(1);
  m_ppComponent[117]->DependsOn(m_ppComponent[1],_Unsigned_);

  m_ppComponent[118]->DependsOn(1);
  m_ppComponent[118]->DependsOn(m_ppComponent[1],_RegWrite_);

  m_ppComponent[119]->DependsOn(1);
  m_ppComponent[119]->DependsOn(m_ppComponent[1],_Link_);

  m_ppComponent[120]->DependsOn(1);
  m_ppComponent[120]->DependsOn(m_ppComponent[1],_RegDst_);

  m_ppComponent[121]->DependsOn(1);
  m_ppComponent[121]->DependsOn(m_ppComponent[119],0);

  m_ppComponent[122]->DependsOn(1);
  m_ppComponent[122]->DependsOn(m_ppComponent[119],0);

  m_ppComponent[123]->DependsOn(1);
  m_ppComponent[123]->DependsOn(m_ppComponent[115],0);

  m_ppComponent[124]->DependsOn(1);
  m_ppComponent[124]->DependsOn(m_ppComponent[115],0);

  m_ppComponent[125]->DependsOn(1);
  m_ppComponent[125]->DependsOn(m_ppComponent[14],0);

  m_ppComponent[126]->DependsOn(1);
  m_ppComponent[126]->DependsOn(m_ppComponent[28],0);

  m_pStart = m_ppComponent[0]; // Cycle starts with Instruction Fetch
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~MonocycleDataPath
//
// DESCRIPTION: Destructor for MonocycleDataPath objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
MonocycleDataPath::~MonocycleDataPath(void)
{
  delete [] m_ppComponent;
  delete [] m_ppCoord;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Init
//
// DESCRIPTION: This method is used to initialize the DataPath Memory.
//
// PARAMETERS: Instruction and Data Memory Allocation Tables.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
MonocycleDataPath::Init(AllocTable* pInstructionAllocTable,
			AllocTable* pDataAllocTable)
{
  // Init Instruction Memory
  ((InstructionMemory*)m_ppComponent[3])->Init(pInstructionAllocTable);
  // Init Data Memory
  ((DataMemory*)m_ppComponent[2])->Init(pDataAllocTable);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Cycle
//
// DESCRIPTION: This method is used to execute an instruction.
//
// PARAMETERS: None.
//
// RETURNS: Updated PC (Word).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
MonocycleDataPath::Cycle(void)
{
  int nComponent;
  // Prepare to start new cycle
  for(nComponent = 0 ; nComponent < m_nComponent ; nComponent++)
    m_ppComponent[nComponent]->NewCycle();
  // Start new cycle
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
            
      m_pStart->GetValue(0,ASK_ACTV); // Instruction execution
      Display();

      break;
    case DISPLAY_TEXT:

      using namespace std;
      cout << endl;
      m_ppComponent[0]->Display();    // Program Counter
      m_ppComponent[4]->Display();    // Registers

      m_pStart->GetValue(0,ASK_ACTV); // Instruction execution

      m_ppComponent[3]->Display();    // Instruction Memory
      m_ppComponent[1]->Display();    // Control Unit
      m_ppComponent[21]->Display();   // ALU
      m_ppComponent[22]->Display();   // SHIFTER
      m_ppComponent[2]->Display();    // Data Memory
      m_ppComponent[0]->Display();    // Program Counter
      m_ppComponent[4]->Display();    // Registers

      break;
    }
  return m_ppComponent[0]->GetValue(0,ASK_PASV);
}

#endif // MONOCYCLEDATAPATH_CPP
