#ifndef PARSER_CPP
#define PARSER_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Parser
//
// BASE CLASSES: N/A.
//
// PURPOSE: Monocycle Parser Test Program.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include <iostream>
#include <iomanip>

#include "../../../Core/Memory.hpp"
 
//Constants
 
//Externals

extern int parse();
extern FILE* yyin;
extern unsigned int yyerrcount;
extern word START_ADDR;
extern word END_ADDR;

//Types and classes
 
Memory yyInstrMem;

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: main
//
// DESCRIPTION: Main Parser function
//
// PARAMETERS:
//  - nArgc : Program argument count
//  - pszArgv : Program arguments
//
// RETURNS: 0 (int).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
int
main(int nArgc,char* pszArgv[])
{
  using namespace std;
  if(nArgc == 2)
    {
      cout << "The KOYOTE Project - Monocycle Parser Version 2.1" << endl;
      cout << "File: " << pszArgv[1] << endl << endl;
      yyin = fopen(pszArgv[1],"rb");
      if(yyin == (FILE*)0)
	{
	  cerr << "Can\'t open file !" << endl << endl;
	}
      else
	{
	  parse();
	  fclose(yyin);
	  if(START_ADDR != END_ADDR)
	    {
	      AllocTable* pAllocTable;
	      cout << hex << setfill('0') << internal << uppercase ;
	      cout << endl << "Instruction Memory content :" << endl;
	      pAllocTable = yyInstrMem.GetAllocTable();
	      while(pAllocTable != (AllocTable*)0)
		{
		  cout << "[0x" << setw(8) << pAllocTable->m_wAddress
		       << "]    0x" << setw(8)
		       << yyInstrMem[pAllocTable->m_wAddress] << endl;
		  pAllocTable = pAllocTable->m_pNext;
		}
	    }
	  else
	    {
	      cout << "Memory empty !" << endl;
	    }
	  if(yyerrcount > 0)
	    {
	      cerr << endl << "Parser: *** [" << pszArgv[1] << "] "
		   << yyerrcount << " error";
	      if(yyerrcount > 1) cerr << "s";
	      cerr << " ***" << endl
		   << "        *** Warning: Instruction Memory"
		   << " initialization incomplete ! ***";
	    }
	  cout << endl;
	}
    }
  else
    cout << pszArgv[0] << " <file>" << endl;
  return 0;
}

#endif // PARSER_CPP
